''Copyright (c) 2009, HMCsoft and Ebben Feagan
''
''All rights reserved.
''
''Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
''
''    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
''    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
''    * Neither the name of HMCsoft nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
''
''THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
''"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
''LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
''A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
''CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
''EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
''PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
''PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
''LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
''NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
''SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'Compiles LOC information of a directory tree for later processing.
'
'Compile with -mt, its mutlithreaded.

' Compile with fbc's '-g' command-line switch to see debugging information

#define fbext_NoBuiltinInstanciations() 1

#include once "ext/misc.bi"
#include once "ext/xml.bi"
#include once "ext/strings.bi"
#include once "ext/debug.bi"
#include once "ext/xstring.bi"
#include once "ext/file/file.bi"
#include once "ext/containers/stack.bi"

const MAX_THREADS = 20

type thread
    as any ptr thread_id
    declare constructor( )
    declare constructor ( byref n_t as const thread )
    declare constructor( byval n_t as any ptr )
    declare destructor( )
end type

constructor thread( )
end constructor

constructor thread( byref n_t as const thread )
    thread_id = n_t.thread_id

end constructor

constructor thread( byval n_t as any ptr )
    thread_id = n_t
end constructor

destructor thread()
    thread_id = 0
end destructor

fbext_Instanciate( fbext_Stack, ((thread)) )

dim shared as any ptr kloc_mutex
dim shared as any ptr print_mutex
dim shared as string base_href
dim shared as integer num_threads
dim shared as uinteger cur_thread

declare sub AnalyzeFile( byval p as any ptr )
declare sub RecurseInto( byval p as any ptr )

using ext
using misc
using strings

sub RecurseInto( byval p as any ptr )

    ' LMF: You were getting warnings because ThreadCreate needs an Any Ptr as
    ' the first parameter of the thread procedure. It's easy to get a typed
    ' pointer tho:
    dim xnode as ext.xml.node ptr = p
    
    ' LMF: This will create an empty stack. fbext_Stack takes a sequence; the
    ' first element is the typename `(thread)`, the second element is the
    ' underlying container to use, `(fbext_Array)` by default. Sequence
    ' elements are enclosed in parenthesis, hence the parenthesis around
    ' `(thread)`. To use a list as the underlying container, use
    ' `fbext_Stack( ((thread)) ((fbext_List)) )`.
    dim threads as fbExt_Stack( ((thread)) )
	dim this_thread_id as uinteger
			mutexlock(kloc_mutex)
		cur_thread += 1
		this_thread_id = cur_thread
		mutexunlock(kloc_mutex)
	do

		mutexlock(kloc_mutex)
		if num_threads < MAX_THREADS then
			num_threads += 1
			mutexunlock(kloc_mutex)
			exit do
		end if

		mutexunlock(kloc_mutex)
		mutexlock(print_mutex)
		FBEXT_DPRINT( this_thread_id & ": Waiting to start..." )
		mutexunlock(print_mutex)
		sleep 50,1
	loop
	
    dim as string this_dir
    mutexlock(kloc_mutex)
	'Have to lock the mutex since the parent could change for some reason
	'Build directory structure

	this_dir = xnode->attribute("name")
	var temp_node = xnode
	while 1
		var par = temp_node->parent()
		if par->tag = "directory" then
			this_dir = par->attribute("name") & "/" & this_dir
		else
			this_dir = base_href & "/" & this_dir
			exit while
		end if
		temp_node = temp_node->parent()
	wend
	
	replace( this_dir, "//", "/" )

    mutexunlock(kloc_mutex)
    mutexlock(print_mutex)
	FBEXT_DPRINT( this_thread_id & ": Currently in: " & this_dir )
	mutexunlock(print_mutex)
	
	if right(this_dir,1) = "/" then
		this_dir = left(this_dir,len(this_dir)-1)
	end if
			
    var dir_count = -1

	
    for n as FILE_ITER = FILE_ITER(this_dir & "/*",&h10) to ""
    
	    var temp = n.filename()

        if ((temp <> "..") AND (temp <> ".") AND (temp <> ".svn")) then
    
            
            var new_dir = xnode->appendChild("directory")
            new_dir->attribute("name") = temp
			'mutexunlock(kloc_mutex)
			mutexlock(print_mutex)
			FBEXT_DPRINT( this_thread_id & ": Found Directory: " & temp )
            mutexunlock(print_mutex)
            dir_count += 1
            
            ' LMF: This will create a temporary thread object constructed with an
            ' Any Ptr, then copy that temporary into the stack memory. All element
            ' type of the standard containers are copied when needed -- just like
            ' C++ -- so make sure that copying is a valid operation, both literally
            ' and semantically, of the types you store.
            threads.Push( ThreadCreate(@RecurseInto,new_dir) )
    
        end if
    next n


	'mutexunlock(kloc_mutex)
	sleep 1,1
	
	
    for n as integer = 0 to dir_count
    
        var file_count = -1
    
        for m as FILE_ITER = FILE_ITER(this_dir & "/*.bas",&h21) to ""
    
            var temp = m.filename()
        
            if ((temp <> "..") AND (temp <> ".") AND (temp <> ".svn")) then
                
                 var new_file = xnode->appendChild("file")
                new_file->attribute("name") = temp
                file_count += 1
                
                AnalyzeFile( new_file )
                 
            end if
    
        next m
        
        
        
        for m as FILE_ITER = FILE_ITER(this_dir & "/*.bi",&h21) to ""
    
            var temp = m.filename()
        
            if ((temp <> "..") AND (temp <> ".") AND (temp <> ".svn")) then
                
                
                var new_file = xnode->appendChild("file")
                new_file->attribute("name") = temp
                file_count += 1
                
                AnalyzeFile( new_file )

        
            end if
        
        next m
    
    next n
    
    do
		if threads.Size() = 0 then exit do
		mutexlock(print_mutex)
		FBEXT_DPRINT(this_thread_id & ": Waiting for children to be done, " & threads.Size() & " left.")
		mutexunlock(print_mutex)
        threadwait threads.Top()->thread_id
        threads.Pop()

    loop
    

	mutexlock(print_mutex)
		FBEXT_DPRINT( this_thread_id & ": Thread Done.")
		mutexunlock(print_mutex)
		    mutexlock(kloc_mutex)
		num_threads -= 1
	mutexunlock(kloc_mutex)
	
end sub

sub AnalyzeFile( byval p as any ptr )

    dim xnode as ext.xml.node ptr = p
    
    static file_con as string
    static num_code as uinteger = 0
    static num_comment as uinteger = 0
    static num_ws as uinteger = 0
    static file_size as ulongint = 0
    static file_name as string
    
    file_con = ""

    
	mutexlock(kloc_mutex)
	'Build directory structure

	var this_dir = xnode->attribute("name")
	var temp_node = xnode
	while 1
		var par = temp_node->parent()
		if par->tag = "directory" then
			this_dir = par->attribute("name") & "/" & this_dir
		else
			this_dir = base_href & "/" & this_dir
			exit while
		end if
		temp_node = temp_node->parent()
	wend
		mutexunlock(kloc_mutex)
	replace( this_dir, "//", "/" )



	if right(this_dir,1) = "/" then
		this_dir = left(this_dir,len(this_dir)-1)
	end if
	
	
	file_name = this_dir
    

    mutexlock(print_mutex)
	FBEXT_DPRINT( "Analyzing File: " & file_name )
	mutexunlock(print_mutex)

	var ff = ext.File(file_name)
	
	
    if ff.open = true then
        file_size = ff.lof
        while not ff.eof
            file_con = ff.linput
            if len(trim(file_con)) > 0 then
                if left(trim(file_con),1) = "'" or left(trim(lcase(file_con)),3) = "rem" then
                    num_comment += 1
                else
                    num_code += 1
                end if
            else
                num_ws += 1
            end if
        wend
		
		else
			mutexlock(print_mutex)
			FBEXT_DPRINT("Unable to open file: " & file_name)
			mutexunlock(print_mutex)
			ff.close
			return
			
	end if
    ff.close
    
    xnode->appendChild( "comments" )
    xnode->child("comments")->attribute("lines") = str(num_comment)
    
    xnode->appendChild( "code" )
    xnode->child("code")->attribute("lines") = str(num_code)
    
    xnode->appendChild( "whitespace" )
    xnode->child("whitespace")->attribute("lines") = str(num_ws)
    
    xnode->appendChild( "filesize" )
    xnode->child("filesize")->attribute("bytes") = str(file_size)

	mutexlock(print_mutex)
	FBEXT_DPRINT( "Done Analyzing: " & file_name )
    mutexunlock(print_mutex)
end sub

kloc_mutex = mutexcreate()
print_mutex = mutexcreate()

dim as xml.tree xtree

cur_thread = 0

FBEXT_DPRINT("Counting lines in: " & curdir())

chdir curdir()

xtree.root->appendChild("ext-kloc")
xtree.root->child("ext-kloc")->attribute("base_href") = curdir
base_href = curdir
xtree.root->child("ext-kloc")->attribute("run_date") = date
xtree.root->child("ext-kloc")->attribute("start_time") = time
xtree.root->child("ext-kloc")->appendChild("directory")'->attribute("name")= base_href

var t_p = threadcreate(@RecurseInto,xtree.root->child("ext-kloc")->child("directory"))

threadwait t_p
scope
var x = xstring(cast(string, xtree))

x.replace(">", ">" & chr(strings.CHAR_LF))

dim xout() as string
x.explode(chr(strings.CHAR_LF), xout() )

open command() for binary ACCESS WRITE as #1

for n as integer = lbound(xout) to ubound(xout)

    print #1, xout(n)

next

FBEXT_DPRINT( "And it only took " & cur_thread & " threads!" )

close
end scope
end 0