#include once "pnginc/png_image.bi"
#include once "crt.bi"

namespace ext.gfx.png
	'::::::::
	' Purpose : filter the current byte
	' Return  : 0 if OK, non zero if unknown filter specified
	function filter_byte _
		( _
		byval pcb       as ubyte ptr, _ ' pointer to byte to be filtered
		byval filter    as integer, _
		byval x         as integer, _
		byval y         as integer, _
		byval scan_size as integer, _
		byval bpp       as integer _
		) as integer

		dim as uinteger a      = 0
		dim as uinteger b      = 0
		dim as uinteger c      = 0
		dim as ubyte    n      = *pcb
		dim as integer  p      = any
		dim as integer  pa     = any
		dim as integer  pb     = any
		dim as integer  pc     = any
		dim as uinteger amount = 0

		function = 0

		select case filter
			case 0

			case 1
				if x >= bpp then
					amount = pcb[-bpp]
				end if
			case 2
				if y > 0 then
					amount = pcb[-scan_size]
				end if
			case 3
				if x >= bpp then
					a = pcb[-bpp]
				end if
				if y > 0 then
					b = pcb[-scan_size]
				end if
				amount = (a + b) shr 1
			case 4
				if x >= bpp then
					a = pcb[-bpp]
				end if
				if y > 0 then
					b = pcb[-scan_size]
					if x >= bpp then
						c = pcb[-(scan_size + bpp)]
					end if
				end if
				' Paeth predictor
				p = a + b - c
				pa = abs( p - a )
				pb = abs( p - b )
				pc = abs( p - c )
				amount = c
				if (pa <= pb) and (pa <= pc) then
					amount = a
				elseif pb <= pc then
					amount = b
				end if
			case else
				function = 1
		end select

		*pcb = n + amount

	end function

	' '::::::::
	' ' Purpose : plot a pixel on an old style freebasic put buffer
	' ' Return  : none
	' sub plot_old _
	' 	( _
	' 		byval p as uinteger ptr, _
	' 		byval x as integer, _
	' 		byval y as integer, _
	' 		byval c as uinteger, _
	' 		byval w as integer, _
	' 		byval h as integer _
	' 	)
	'
	' 	p = cptr( uinteger ptr, @cptr( ubyte ptr, p )[sizeof( OLD_HEADER )] )
	' 	p[x + (y * w)] = c
	'
	' end sub

	'::::::::
	' Purpose : plot a pixel on a new style freebasic put buffer
	' Return  : none
	sub plot_new _
		( _
		byval p as uinteger ptr, _
		byval x as integer, _
		byval y as integer, _
		byval c as uinteger, _
		byval w as integer, _
		byval h as integer _
		)

		dim as integer offset = any

		offset = sizeof( NEW_HEADER ) + (y * cptr( NEW_HEADER ptr, p )->pitch)
		p = cptr( uinteger ptr, @cptr( ubyte ptr, p )[offset] )
		p[x] = c

	end sub

	'::::::::
	' Purpose : plot a pixel on a buffer suitable for OpenGL
	' Return  : none
	sub plot_opengl _
		( _
		byval p as uinteger ptr, _
		byval x as integer, _
		byval y as integer, _
		byval c as uinteger, _
		byval w as integer, _
		byval h as integer _
		)

		c = (cuint( (c shr 16) and &hff ) shl  0) _
		or (cuint( (c shr  8) and &hff ) shl  8) _
		or (cuint( (c shr  0) and &hff ) shl 16) _
		or (cuint( (c shr 24) and &hff ) shl 24)
		p[x + ((h - y - 1) * w)] = c

	end sub

	'::::::::
	' Purpose : Enough data has been seen, so pixel(s) can now be output
	' Return  : none
	sub png_image_draw_pixels _
		( _
		byref png_image as png_image_t, _
		byval out_img   as any ptr, _
		byval p         as ubyte ptr, _
		byval plot      as plot_func_t, _
		byref x1        as integer, _
		byval y1        as integer, _
		byval wfactor   as integer, _
		byval hfactor   as integer _
		)

		dim as uinteger c      = any
		dim as integer  i      = any
		dim as ubyte    alpha  = PNG_DEFAULT_ALPHA
		dim as integer  max    = any
		dim as integer  mask   = any

		with png_image

			if .colortype = 0 then
				if .bitdepth <> 16 then
					max  = (8 \ .bitdepth) - 1
					mask = (2 ^ .bitdepth) - 1
					for i = 0 to max
						if x1 < .width then
							c = (*p shr ((max - i) * .bitdepth)) and mask
							alpha = PNG_DEFAULT_ALPHA
							if .has_tRNS and (c = .tRNS_0) then
								alpha = 0
							end if
							c *= 255 \ mask
							c = RGBA( c, c, c, alpha )
							plot( out_img, x1, y1, c, .width, .height )
						end if
						x1 += wfactor
					next i
				else
					c = get_u16( @p[-1] )
					if .has_tRNS and (c = .tRNS_0) then
						alpha = 0
					end if
					c shr= 8
					c = RGBA( c, c, c, alpha )
					plot( out_img, x1, y1, c, .width, .height )
					x1 += wfactor
				end if
			elseif .colortype = 3 then
				dim as integer max    = (8 \ .bitdepth) - 1
				dim as integer mask   = (2 ^ .bitdepth) - 1
				for i = 0 to max
					if x1 < .width then
						c = (*p shr ((max - i) * .bitdepth)) and mask
						alpha = PNG_DEFAULT_ALPHA
						if .has_tRNS then
							alpha = .tRNS_3(c)
						end if
						with .PLTE(c)
							c = RGBA( .r, .g, .b, alpha )
						end with
						plot( out_img, x1, y1, c, .width, .height )
					end if
					x1 += wfactor
				next i
			elseif .colortype = 4 then
				if .bitdepth = 8 then
					c = p[-1]
					alpha = *p
				else
					c = p[-3]
					alpha = p[-1]
				end if
				c = RGBA( c, c, c, alpha )
				plot( out_img, x1, y1, c, .width, .height )
				x1 += wfactor
			elseif (.colortype = 2) or (.colortype = 6) then
				if .bitdepth = 8 then
					if .bpp = 3 then
						if .has_tRNS = TRUE then
							if (p[-2] = .tRNS_2r) and (p[-1] = .tRNS_2g) and (*p = .tRNS_2b) then
								alpha = 0
							end if
						end if
						c = RGBA( p[-2], p[-1], *p, alpha )
					else
						c = RGBA( p[-3], p[-2], p[-1], *p )
					end if
					plot( out_img, x1, y1, c, .width, .height )
					x1 += wfactor
				else
					dim as ushort r2 = any
					dim as ushort g2 = any
					dim as ushort b2 = any
					if .bpp = 6 then
						r2 = get_u16( @p[-5] )
						g2 = get_u16( @p[-3] )
						b2 = get_u16( @p[-1] )
						if .has_tRNS = TRUE then
							if (r2 = .tRNS_2r) and (g2 = .tRNS_2g) and (b2 = .tRNS_2b) then
								alpha = 0
							end if
						end if
					else
						r2 = get_u16( @p[-7] )
						g2 = get_u16( @p[-5] )
						b2 = get_u16( @p[-3] )
						alpha = p[-1]
					end if
					c = RGBA( r2 shr 8, g2 shr 8, b2 shr 8, alpha )
					plot( out_img, x1, y1, c, .width, .height )
					x1 += wfactor
				end if
			end if

		end with

	end sub

	# define FBEXT_CALC_IMAGE_PITCH(w, bitdepth) (((w) * (bitdepth) + &HF) and not &HF)' ((((w) * (bitdepth / 8)) + 31) and &hffffffe0)
	# define FBEXT_CALC_IMAGE_BUFFERSIZE(w, h, bitdepth) (header_size + (FBEXT_CALC_IMAGE_PITCH(w, bitdepth) * (h)))

	'' :::::
	private function GetImageBuffer_ ( byval target as target_e, byval w as SizeType, byval h as SizeType, byval bitdepth as SizeType ) as any ptr

		function = null

		if (target = TARGET_OPENGL) then

			return allocate( w * h * (bitdepth / 4) )

		else

			return imagecreate( w, h, , bitdepth )

		end if

	end function

	'' :::::
	private sub PutImageBuffer_ ( byval img as any ptr )
		delete cast(ubyte ptr, img)
	end sub

	'' :::::
	private sub ImageConvert_ ( byval src as fb.Image ptr, byref dst as fb.Image ptr, byval bitdepth as SizeType )

		if (src = null) then
			DEBUGPRINT( "null source image buffer" )
			dst = null
			exit sub
		end if

		dim src_width as const SizeType = src->width
		dim src_pitch as const SizeType = src->pitch
		dim src_bitdepth as const SizeType = src->bpp * 8
		dim dst_pitch as SizeType = src_width * (bitdepth / 8)

		' new-style buffer ?
		if (src->type = PUT_HEADER_NEW) then
			dst = GetImageBuffer_( TARGET_FBNEW, src_width, src->height, bitdepth )
			dst_pitch = FBEXT_CALC_IMAGE_PITCH(src_width, bitdepth)

		end if

		var src_row = cast(ubyte ptr, src) + sizeof(FB.IMAGE)
		var dst_row = cast(ubyte ptr, dst) + sizeof(FB.IMAGE)

		for row as SizeType = 0 to src->height - 1
			..ImageConvertRow(              _
			src_row, src_bitdepth,      _
			dst_row, bitdepth,          _
			src_width, 0)

			src_row += src_pitch
			dst_row += dst_pitch
		next
	end sub

	'' ///
	'' /// /lmf
	'' ///

	'::::::::
	' Purpose : Create an image buffer of requested target type, and unpack the IDAT
	' Return  : NULL on failure, valid pointer to buffer on success
	function png_image_convert _
		( _
		byref png_image as png_image_t, _
		byval target    as target_e _
		) as any ptr

		dim as integer      offset    = 0
		dim as integer      data_read = 0
		dim as any ptr out_img = any
		dim as ubyte ptr    in_img    = any
		dim as integer      pass_max  = 1
		dim as plot_func_t  plot      = NULL
		dim as integer      pass      = any

		function = NULL

		with png_image

			if .prepared = FALSE then
				DEBUGPRINT( "Not prepared" )
				exit function
			end if

			' Make a copy of the IDAT, because unfiltering will overwrite
			in_img = callocate( .IDAT_len )
			if in_img = NULL then
				DEBUGPRINT( "IDAT copy allocate failed" )
				exit function
			end if
			memcpy( in_img, .IDAT, .IDAT_len )

			' creates only 32 bits-per-pixel buffers..
			select case as const target

				case TARGET_FBNEW
					plot = @plot_new

				case TARGET_OPENGL
					plot = @plot_opengl

				case else
					deallocate( in_img )
					DEBUGPRINT( "Bad target" )
					exit function
			end select

			out_img = GetImageBuffer_(target, .width, .height, 32)
			if out_img = NULL then
				deallocate( in_img )
				DEBUGPRINT( "Output buffer allocate failed" )
				exit function
			end if

			if .interlacemethod = 1 then
				pass_max = 7
			end if

			' Make a number of passes, 1 for non interlaced, 7 for interlaced
			for pass = 1 to pass_max
				dim as integer scan_size = calc_scan_size( png_image, pass )
				dim as integer y1        = 0
				dim as integer wfactor   = 1
				dim as integer hfactor   = 1
				dim as integer y_max     = .height - 1
				dim as integer y         = any
				if .interlacemethod = 1 then
					dim as integer iheight = any
					wfactor = tb_wfac(pass)
					hfactor = tb_hfac(pass)
					iheight = (.height + hfactor - tb_yoff(pass) - 1) \ hfactor
					y1 = tb_yoff(pass)
					y_max = iheight - 1
					' Force the y loop to not do anything if theres nothing to do this pass
					if .width <= tb_xoff(pass) then
						y_max = -1
					end if
				end if
				for y = 0 to y_max
					dim as integer x1     = 0
					dim as ubyte   filter = in_img[offset]
					dim as integer x      = any
					offset += 1 ' Jump past the filter byte
					if .interlacemethod = 1 then
						x1 = tb_xoff(pass)
					end if
					for x = 0 to scan_size - 2
						' filter one byte, not pixel
						if filter_byte( @in_img[offset], filter, x, y, scan_size, .bpp ) <> 0 then
							PutImageBuffer_( out_img )
							deallocate( in_img )
							DEBUGPRINT( "Bad filter byte" )
							exit function
						end if
						data_read += 1 ' Count bytes until theres enough been filtered
						if data_read = .bpp then ' to be able to draw some
							data_read = 0
							png_image_draw_pixels(                      _
							png_image, out_img, @in_img[offset],    _
							plot, x1, y1, wfactor, hfactor )
						end if
						offset += 1
					next x
					y1 += hfactor
				next y
			next pass

			deallocate( in_img )
		end with

		var scrn_bitdepth = 0
		scope
		var unused = 0
		screeninfo unused, unused, scrn_bitdepth
		end scope

		if (target = TARGET_OPENGL or (scrn_bitdepth = 32)) then
			function = out_img

		else
			dim new_img as fb.Image ptr = any
			ImageConvert_(out_img, new_img, scrn_bitdepth)
			PutImageBuffer_(out_img)

			function = new_img
		end if

	end function

end namespace 'ext.gfx.png
