''Copyright (c) 2009, FreeBASIC Extended Library Development Group
''
''All rights reserved.
''
''Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
''
''    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
''    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
''    * Neither the name of the FreeBASIC Extended Library Development Group nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
''
''THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
''"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
''LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
''A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
''CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
''EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
''PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
''PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
''LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
''NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
''SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# include once "ext/strings.bi"
# include once "ext/misc.bi"
# include once "crt/string.bi"

namespace ext.strings

	'' :::::
	sub Replace (byref subject as string, byref oldtext as const string, byref newtext as const string)
	
		if 0 = len(subject) then exit sub
		if 0 = len(oldtext) then exit sub
		
        select case len(newtext)
        
        ' subject remains the same size ?
        case is = len(oldtext)
            var dst = strptr(subject)
            
			' work in-place, copying replacements as they are found.
			var strpos = instr(subject, oldtext)
			do while strpos > 0
				..memcpy(dst + strpos - 1, strptr(newtext), len(newtext))
				
				strpos = instr(strpos + len(oldtext), subject, oldtext)
			loop
		
        ' subject needs to shrink ?
        case is < len(oldtext)
        	var replacements = 0
			var src = strptr(subject)
			var dst = strptr(subject)
			
			' work in-place, copying arbitrary characters and replacements
			' in turn.
			var strpos = instr(subject, oldtext)
			do while strpos > 0
				replacements += 1
				
				var chars_before = @subject[strpos - 1] - src
				if chars_before > 0 then
				    ' not at the beginning ?
				    if dst <> src then
    					..memcpy(dst, src, chars_before)
    		        end if
					src += chars_before
					dst += chars_before
				end if
				
				..memcpy(dst, strptr(newtext), len(newtext))
				src += len(oldtext)
				dst += len(newtext)
				
				strpos = instr(strpos + len(oldtext), subject, oldtext)
			loop
			
			' copy remaining text, if any.
			var chars_after = @subject[len(subject)] - src
			if chars_after > 0 then
				..memcpy(dst, src, chars_after)
			end if
			
			' matches FB's internal string descriptor structure.
			type FBStringDesc__
				chardata		as zstring ptr
				length			as integer
				capacity		as integer
			end type
			
			' just change the size, capacity remains the same.
			var desc = cast(FBStringDesc__ ptr, @subject)
			var newsize = len(subject) + replacements * (len(newtext) - len(oldtext))
			desc->length = newsize
			(*desc->chardata)[newsize] = 0
		
        ' subject needs to grow ?
		case else
            static s_reptable as string
            const s_init_reptable_size = 1024
            
            if len(s_reptable) = 0 then
                s_reptable = space(s_init_reptable_size * sizeof(ext.SizeType))
            end if
            
            # define REPTABLE_SIZE() cast(ext.SizeType, len(s_reptable) / sizeof(ext.SizeType))
            # define REPTABLE_GETREP(index_) *cast(ext.SizeType ptr, @s_reptable[index_ * sizeof(ext.SizeType)])
            
            var replacements = 0
            
            ' find and store the positions of any replacements.
            var strpos = instr(subject, oldtext)
            do while strpos > 0
                ASSERT( replacements <= REPTABLE_SIZE() )
                
                if replacements = REPTABLE_SIZE() then
                    s_reptable &= space(sizeof(ext.SizeType))
                end if
                
                REPTABLE_GETREP(replacements) = strpos
                replacements += 1
                
	            strpos = instr(strpos + len(oldtext), subject, oldtext)
            loop
            
            ' grow subject.
            var mem_needed = replacements * (len(newtext) - len(oldtext))
            var old_len = len(subject)
            subject &= space(mem_needed)
            
    		var src = strptr(subject) + old_len
    		var dst = strptr(subject) + len(subject)
    		
            for i as integer = replacements - 1 to 0 step -1
                ' copy after replacement.
                var chars_after = src - @subject[REPTABLE_GETREP(i) - 1] - len(oldtext)
                if chars_after > 0 then
                    src -= chars_after
                    dst -= chars_after
                    ..memcpy(dst, src, chars_after)
                end if
                
                ' copy replacement.
                src -= len(oldtext)
                dst -= len(newtext)
                ..memcpy(dst, strptr(newtext), len(newtext))
            next
		
        end select
	
	end sub

	'' :::::
	sub Replace (byref subject as string, oldtext() as const string, byref newtext as const string)

		for i as integer = lbound(oldtext) to ubound(oldtext)
			ext.strings.Replace(subject, oldtext(i), newtext)
		next

	end sub


	'' :::::
	sub Replace (byref subject as string, oldtext() as const string, newtext() as const string)

		var result = subject

		var search_size = ubound(oldtext) - lbound(oldtext) + 1
		var replace_size = ubound(newtext) - lbound(newtext) + 1

		var n = FBEXT_MIN(search_size, replace_size)

		for i as integer = lbound(oldtext) to lbound(oldtext) + n - 1
			ext.strings.Replace(subject, oldtext(i), newtext(i))
		next

		if search_size > replace_size then
			for i as integer = n - 1 to ubound(oldtext)
				ext.strings.Replace(subject, oldtext(i), "")
			next
		end if

	end sub

	'' :::::
	sub replace (subject() as string, byref oldtext as const string, byref newtext as const string)

		for i as integer = lbound(subject) to ubound(subject)
			ext.strings.Replace(subject(i), oldtext, newtext)
		next

	end sub

	'' :::::
	sub replace (subject() as string, oldtext() as const string, byref newtext as const string)

		for i as integer = lbound(subject) to ubound(subject)
			ext.strings.Replace(subject(i), oldtext(), newtext)
		next

	end sub

	'' :::::
	sub replace (subject() as string, oldtext() as const string, newtext() as const string)

		for i as integer = lbound(subject) to ubound(subject)
			replace(subject(i), oldtext(), newtext())
		next

	end sub

end namespace
