''Copyright (c) 2009, FreeBASIC Extended Library Development Group
''
''All rights reserved.
''
''Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
''
''    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
''    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
''    * Neither the name of the FreeBASIC Extended Library Development Group nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
''
''THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
''"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
''LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
''A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
''CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
''EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
''PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
''PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
''LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
''NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
''SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include once "ext/detail/common.bi"
#include once "ext/xml.bi"

#include once "crt.bi"
#undef NULL

'' zero-terminated strncpy
private function strnztcpy(byval dest as ubyte ptr, byval src as ubyte ptr, byval n as size_t) as ubyte ptr
	dest[n] = 0
	return strncpy(dest, src, n)
end function

namespace ext.xml

	function encode_entities(byref text as const string) as string
		dim as string temp
		for i as integer = 0 to len(text) - 1
			select case text[i]
			case quot
				temp &= "&quot;"
			case amp
				temp &= "&amp;"
			case apos
				temp &= "&apos;"
			case lt
				temp &= "&lt;"
			case gt
				temp &= "&gt;"
			case else
				if text[i] < space_ and not (text[i] = tab_ or text[i] = lf or text[i] = cr) then
					temp &= "&#" & text[i] & ";"
				elseif text[i] and &b10000000 then
					dim as integer u
					i += decode_utf8(@text[i], u) - 1
					temp &= "&#" & u & ";"
				else
					temp &= chr(text[i])
				end if
			end select
		next
		return temp
	end function

	function decode_entities(byref text as const string) as string
		dim as string temp
		dim as integer i = 1, j
		do
			j = instr(i, text, "&")
			if (j) then
				temp &= mid(text, i, j - i)
				i = j + 1
				j = instr(i, text, ";")
				if (j) then
					select case (mid(text, i, j - i + 1))
					case "quot;"
						temp &= """"
					case "amp;"
						temp &= "&"
					case "apos;"
						temp &= "'"
					case "lt;"
						temp &= "<"
					case "gt;"
						temp &= ">"
					case else
						if text[i - 1] = hash then
							dim as integer u
							if text[i] = asc("x") then
								u = val("&h" & mid(text, i + 2, j - i))
							else
								u = val(mid(text, i + 1, j - i))
							end if
							if (u) then
								dim as zstring ptr encoded = encode_utf8(u)
								if (encoded) then
									temp &= *encoded
									deallocate(encoded)
								end if
							end if
						else
							temp &= "&"
							j = i - 1
						end if
					end select
					i += j - i + 1
				end if
			end if
		loop while (j)
		return temp & mid(text, i)
	end function

	function encode_utf8(byval u as integer) as zstring ptr
		dim as integer n

		if     (u >= &h010000 and u <= &h10ffff) then
			n = 4
		elseif (u >= &h000800 and u <= &h00ffff) then
			n = 3
		elseif (u >= &h000080 and u <= &h0007ff) then
			n = 2
		elseif (u >= &h000000 and u <= &h00007f) then
			n = 1
		else  '' not a UTF-8 character
			return 0
		end if

		dim as zstring ptr dst = callocate(n + 1)

		if (n > 1) then
			for j as integer = 0 to n - 1
				dst[n - 1 - j] = u and iif(j = n - 1, 1 shl (7 - n), 1 shl 6) - 1
				dst[n - 1 - j] or= iif(j = n - 1, not ((1 shl (8 - n)) - 1), &b10000000)
				u shr= 6
			next
		else
			dst[0] = u
		end if

		return dst
	end function

	function decode_utf8(byval src as zstring ptr, byref u as integer) as integer
		dim as integer n

		if     (src[0] and &b11111000) = &b11110000 then
			n = 4
		elseif (src[0] and &b11110000) = &b11100000 then
			n = 3
		elseif (src[0] and &b11100000) = &b11000000 then
			n = 2
		elseif (src[0] and &b10000000) = &b00000000 then
			n = 0
		else  '' not a UTF-8 character
			n = 0
		end if

		if (n) then
			u = 0

			for j as integer = 0 to n - 1
				if (j <> 0) and (src[j] and &b11000000) <> &b10000000 then
					goto not_utf8
				end if

				if ((src[j] and &b11111110) = &b11000000 or _
				(src[j]               ) = &b11110101 or _
				(src[j] and &b11111110) = &b11110110 or _
				(src[j] and &b11111100) = &b11111000 or _
				(src[j] and &b11111110) = &b11111100 or _
				(src[j] and &b11111110) = &b11111110) then
					goto not_utf8
				end if

				u shl= 6
				u or= src[j] and (iif(j = 0, 1 shl (7 - n), 1 shl 6) - 1)
			next
		else
			not_utf8:
			n = 1
			u = src[0]
		end if

		return n
	end function


	function node.m_child_index(byref tag_ as const string, byval index as integer) as integer
		'' search for the child
		dim as integer i = 0
		do while (i < m_children)
			if (m_child[i]->tag = tag_) then
				if (index = 0) then
					return i
				end if
				index -= 1
			end if
			i += 1
		loop
		'' child not found
		return -1
	end function

	function node.Parent( ) as node ptr
	
		return m_parent
	
	end function	

	
	function node.m_child_index(byval node_type_ as node_type_e, byval index as integer) as integer
		'' search for the child
		dim as integer i = 0
		do while (i < m_children)
			if (m_child[i]->nodeType = node_type_) then
				if (index = 0) then
					return i
				end if
				index -= 1
			end if
			i += 1
		loop
		'' child not found
		return -1
	end function

	function node.m_emit() as string
		'' not finished
		select case m_node_type
		case element
			dim as string temp
			temp = "<" & tag
			if (m_attributes) then
				for i as integer = 0 to m_attributes - 1
					temp &= " " & *m_attribute[i] & "=""" & encode_entities(*m_value[i]) & """"
				next
			end if
			if (m_children) then
				temp &= ">"
				for i as integer = 0 to m_children - 1
					temp &= m_child[i]->m_emit()
				next
				temp &= "</" & tag & ">"
			else
				temp &= " />"
			end if
			return temp
		case comment
			return "<!--" & attribute("") & "-->"
		case text
			return encode_entities(attribute(""))
		case processingInstruction
			return "<?" & tag & attribute("") & "?>"
		case declaration
			return "<!" & tag & attribute("") & ">"
		end select
	end function

	sub node.m_parse(byref xmltext as const string)
		dim as node ptr root = @this
		dim as node ptr current = root

		dim as integer i = 0, l = len(xmltext) - 1

		do until i > l
			if xmltext[i] = lt then
				i += 1
				if (xmltext[i] = exclam and xmltext[i + 1] = hyphen and xmltext[i + 2] = hyphen) then
					i += 3

					'' grab the comment
					dim as integer j = i
					do until (xmltext[i] = hyphen and xmltext[i + 1] = hyphen and xmltext[i + 2] = gt) or i > l
						i += 1
					loop

					if (i - j) then
						dim as zstring ptr tempcomment = allocate(i - j + 1)
						strnztcpy(tempcomment, @xmltext[j], i - j)

						'' create the comment child
						current->appendChild("", comment)
						*(current->m_child[current->m_children - 1]) = *tempcomment

						deallocate(tempcomment)
					else
						'' empty comment?
						current->appendChild("", comment)
					end if

					i += 3

				elseif (xmltext[i] = quest) then
					i += 1

					'' grab the tag
					dim as integer j = i
					do until FBEXT_XML_IS_WHITESPACE(xmltext[i]) or (xmltext[i] = quest and xmltext[i + 1] = gt) or i > l
						i += 1
					loop

					if (i - j) then
						dim as zstring ptr temptag = allocate(i - j + 1)
						strnztcpy(temptag, @xmltext[j], i - j)

						'' create the child
						current->appendChild(*temptag, processingInstruction)

						deallocate(temptag)
					else
						'' MALFORMED
						current->appendChild("", processingInstruction)
					end if

					'' grab the instructions
					dim as integer k = i
					do until (xmltext[i] = quest and xmltext[i + 1] = gt) or i > l
						i += 1
					loop

					if (i - k) then
						dim as zstring ptr temptext = allocate(i - k + 1)
						strnztcpy(temptext, @xmltext[k], i - k)

						'' set the instructions
						*current->m_child[current->m_children - 1] = *temptext

						deallocate(temptext)
					end if

					i += 2

				elseif (xmltext[i] = exclam) then '' CDATA? halp
					i += 1

					'' grab the tag
					dim as integer j = i
					do until FBEXT_XML_IS_WHITESPACE(xmltext[i]) or xmltext[i] = gt or i > l
						i += 1
					loop

					if (i - j) then
						dim as zstring ptr temptag = allocate(i - j + 1)
						strnztcpy(temptag, @xmltext[j], i - j)

						'' create the child
						current->appendChild(*temptag, declaration)

						deallocate(temptag)
					else
						'' MALFORMED
						current->appendChild("", declaration)
					end if

					'' grab the instructions
					dim as integer k = i
					do until xmltext[i] = gt or i > l
						i += 1
					loop

					if (i - k) then
						dim as zstring ptr temptext = allocate(i - k + 1)
						strnztcpy(temptext, @xmltext[k], i - k)

						'' set the instructions
						*current->m_child[current->m_children - 1] = *temptext

						deallocate(temptext)
					end if

					i += 2

				elseif (xmltext[i] = slash) then
					i += 1

					'' grab the tag
					dim as integer j = i
					do until xmltext[i] = gt or FBEXT_XML_IS_WHITESPACE(xmltext[i]) or i > l
						i += 1
					loop

					if (i - j) then
						dim as zstring ptr temptag = allocate(i - j + 1)
						strnztcpy(temptag, @xmltext[j], i - j)

						'' run up the tree until the tag being closed is found
						dim as node ptr depth = current
						do until depth = root
							if (depth->tag = *temptag) then
								current = depth->m_parent
								exit do
							end if
							depth = depth->m_parent
						loop

						deallocate(temptag)
					else
						/' new-fangled xml? '/
					end if

					'' get rid of the rest (should only be spaces, otherwise MALFORMED)
					do until xmltext[i] = gt or i > l
						i += 1
					loop

					i += 1

				else
					'' grab the child tag
					dim as integer j = i
					do until xmltext[i] = gt or FBEXT_XML_IS_WHITESPACE(xmltext[i]) or xmltext[i] = slash or i > l
						i += 1
					loop

					if (i - j) then
						dim as zstring ptr temptag = allocate(i - j + 1)
						strnztcpy(temptag, @xmltext[j], i - j)

						'' create the child
						current = current->appendChild(*temptag)

						deallocate(temptag)
					else
						'' MALFORMED
						current = current->appendChild("")
					end if

					'' grab attributes until the end of the tag
					do until xmltext[i] = gt or (xmltext[i] = slash and xmltext[i + 1] = gt) or i > l
						dim as zstring ptr tempattribute = 0, tempvalue = 0
						if not FBEXT_XML_IS_WHITESPACE(xmltext[i]) then

							'' grab the attribute
							dim as integer j = i
							do until xmltext[i] = eq or FBEXT_XML_IS_WHITESPACE(xmltext[i]) or xmltext[i] = gt or i > l
								i += 1
							loop

							if (i - j) then
								tempattribute = allocate(i - j + 1)
								strnztcpy(tempattribute, @xmltext[j], i - j)
							end if

							'' grab the value
							if xmltext[i] = eq then
								i += 1

								'' if value is surrounded by quotes
								if xmltext[i] = quot then
									i += 1

									dim as integer j = i
									do until xmltext[i] = quot or i > l
										i += 1
									loop

									if (i - j) then
										tempvalue = allocate(i - j + 1)
										strnztcpy(tempvalue, @xmltext[j], i - j)
									end if

									i += 1

								'' if value is not
								else

									dim as integer j = i
									do until FBEXT_XML_IS_WHITESPACE(xmltext[i]) or xmltext[i] = gt or i > l
										i += 1
									loop

									if (i - j) then
										tempvalue = allocate(i - j + 1)
										strnztcpy(tempvalue, @xmltext[j], i - j)
									end if

								end if
							end if

							'' set the attribute
							if (tempattribute) then
								if (tempvalue) then
									current->attribute(*tempattribute) = decode_entities(*tempvalue)
									deallocate(tempvalue)
								else
									current->attribute(*tempattribute) = ""
								end if
								deallocate(tempattribute)
							end if

						else
							'' skip the whitespace
							i += 1
						end if
					loop

					'' self-close tag
					if (xmltext[i] = slash and xmltext[i + 1] = gt) then
						current = current->m_parent
						i += 2
					else
						i += 1
					end if

				end if
			else
				'' this is so hackish, it's awesome
				' if iif(current->m_children, current->m_child[current->m_children - 1]->nodeType = text, 0) = 0 then
					current->appendChild("", text)
				' end if

				'' grab the text
				dim as integer j = i
				do until xmltext[i] = lt or i > l
					i += 1
				loop

				if (i - j) then
					dim as zstring ptr temptext = allocate(i - j + 1)
					strnztcpy(temptext, @xmltext[j], i - j)

					'' set the text
					*current->m_child[current->m_children - 1] = decode_entities(*temptext)

					deallocate(temptext)
				end if

			end if
		loop

	end sub

	constructor node
	end constructor

	destructor node
		if (m_attributes) then
			for i as integer = 0 to m_attributes - 1
				deallocate(m_attribute[i])
				deallocate(m_value[i])
			next
			deallocate(m_attribute)
			deallocate(m_value)
		end if
		do while (m_children)
			removeChild(0)
		loop
	end destructor

	operator node.let(byref new_value as const string)
		'' not finished
		select case m_node_type
		case element
			m_parse(new_value)
		case comment, text, processingInstruction, declaration
			attribute("") = new_value
		end select
	end operator

	operator node.cast() as string
		return m_emit()
	end operator

	function node.nodeType as node_type_e
		return m_node_type
	end function

	property node.getText( ) as string
		if m_children > 0 then

			return child("")->attribute("")

		end if

		return ""


	end property

	property node.setText( byref text_ as const string )
		if m_children > 0 then
			child("")->attribute("") = text_
		end if
	end property

	property node.attribute(byref attribute_ as const string) as string
		'' return the attribute if it exists
		if (m_attributes) then
			for i as integer = 0 to m_attributes - 1
				if (*m_attribute[i] = attribute_) then
					return *m_value[i]
				end if
			next
		end if
		'' return nothing if it doesn't
		return ""
	end property

	property node.attribute(byref attribute_ as const string, byref new_value as const string)
		'' modify the attribute if it exists
		if (m_attributes) then
			for i as integer = 0 to m_attributes - 1
				if (*m_attribute[i] = attribute_) then
					m_value[i] = reallocate(m_value[i], len(new_value) + 1)
					*m_value[i] = new_value
					return
				end if
			next
		end if
		'' add the attribute if it doesn't
		m_attributes += 1
		m_attribute = reallocate(m_attribute, m_attributes * sizeof(zstring ptr))
		m_value = reallocate(m_value, m_attributes * sizeof(zstring ptr))
		m_attribute[m_attributes - 1] = allocate(len(attribute_) + 1)
		m_value[m_attributes - 1] = allocate(len(new_value) + 1)
		*m_attribute[m_attributes - 1] = attribute_
		*m_value[m_attributes - 1] = new_value
	end property

	function node.children() as integer
		return m_children
	end function

	function node.children(byref tag_ as const string) as integer
		dim as integer i = 0, c = 0
		do while (i < m_children)
			if (m_child[i]->tag = tag_) then
				c += 1
			end if
			i += 1
		loop
		return c
	end function

	function node.children(byval node_type_ as node_type_e) as integer
		dim as integer i = 0, c = 0
		do while (i < m_children)
			if (m_child[i]->nodeType = node_type_) then
				c += 1
			end if
			i += 1
		loop
		return c
	end function

	function node.child(byval index as integer = 0) as node ptr
		'' does that child exist?
		if (index < m_children) then
			return m_child[index]
		end if
		'' child does not exist
		return 0
	end function

	function node.child(byref tag_ as const string, byval index as integer = 0) as node ptr
		dim as integer i = m_child_index(tag_, index)
		if (i = -1) then
			'' child does not exist
			return 0
		end if
		return m_child[i]
	end function

	function node.child(byval node_type_ as node_type_e, byval index as integer = 0) as node ptr
		dim as integer i = m_child_index(node_type_, index)
		if (i = -1) then
			'' child does not exist
			return 0
		end if
		return m_child[i]
	end function

	function node.lastChild() as node ptr
		if (m_children) then
			return m_child[m_children - 1]
		end if
		return 0
	end function

	function node.appendChild(byref tag_ as const string, byval node_type_ as node_type_e = element) as node ptr
		m_children += 1
		m_child = reallocate(m_child, m_children * sizeof(node ptr))
		m_child[m_children - 1] = new node
		m_child[m_children - 1]->m_node_type = node_type_
		m_child[m_children - 1]->tag = tag_
		m_child[m_children - 1]->m_parent = @this
		return m_child[m_children - 1]
	end function

	sub node.removeChild(byval index as integer)
		dim as node ptr tempchild = child(index)
		'' if child exists
		if (tempchild) then
			'' shift the children up
			if (index < m_children - 1) then
				for i as integer = index to m_children - 2
					swap m_child[i + 1], m_child[i]
				next
			end if
			m_children -= 1
			m_child = reallocate(m_child, m_children * sizeof(node ptr))

			delete tempchild
		end if
	end sub

	sub node.removeChild(byref tag_ as const string, byval index as integer = 0)
		dim as integer i = m_child_index(tag_, index)
		if (i = -1) then
			'' child does not exist
			return
		end if
		removeChild(i)
	end sub

	sub node.removeChild(byval node_type_ as node_type_e, byval index as integer = 0)
		dim as integer i = m_child_index(node_type_, index)
		if (i = -1) then
			'' child does not exist
			return
		end if
		removeChild(i)
	end sub


	constructor tree
		root = new node
	end constructor

	destructor tree
		if (root) then
			do while root->children
				root->removeChild(0)
			loop
		end if
		delete root
	end destructor

	operator tree.let(byref new_value as const string)
		*root = new_value
	end operator

	operator tree.cast as string
		dim as string temp
		for i as integer = 0 to root->children - 1
			temp &= *root->child(i)
		next
		return temp
	end operator

	function tree.load(byref xmlfile as const string) as integer
		dim as integer f = freefile
		if (open(xmlfile for binary access read as #f) = 0) then
			dim as zstring ptr tempxml = callocate(sizeof(ubyte) * lof(f) + 1)
			get #f, , *cast(ubyte ptr, tempxml), lof(f)
			this = *tempxml
			deallocate(tempxml)
			close #f
			return 0
		end if
		return -1
	end function

	function tree.unload(byref xmlfile as const string) as integer
		dim as integer f = freefile
		if (open(xmlfile for binary access write as #f) = 0) then
			put #f, , cast(string, this)
			close #f
			return 0
		end if
		return -1
	end function

end namespace 'ext.xml
