# include once "ext/testly.bi"
# include once "ext/hash.bi"

namespace ext.tests.hashes

	const FBEXT_TEST_STRING = "FreeBASIC Extended Library Testing Testing 123"

	private sub test_adler32
	
		var res = ext.hashes.adler32( FBEXT_TEST_STRING )

		TESTLY_ASSERT_TRUE( 2001539036u = res )
	
	end sub
	
	private sub test_crc32

		var res = ext.hashes.crc32( FBEXT_TEST_STRING )
	
		TESTLY_ASSERT_TRUE( 881431911u = res )
	
	end sub
	
	private sub test_joaat32
	
		var res = ext.hashes.joaat( FBEXT_TEST_STRING )

		TESTLY_ASSERT_TRUE( 2896898767u = res )
	
	end sub

	private sub test_joaat64

		var res = ext.hashes.joaat64( FBEXT_TEST_STRING )
	
		TESTLY_ASSERT_TRUE( 4218073866758198310ull = res )
	
	end sub
	
	private function run_test_md5( byref in_ as const string ) as string
	
		dim as ext.hashes.md5.state m_state
		
		ext.hashes.md5.init( @m_state )
		ext.hashes.md5.append( @m_state, @(in_[0]), len(in_) )

		return ext.hashes.md5.finish( @m_state )
	
	end function
	
	private sub test_md5
										  
	TESTLY_ASSERT_STRING_EQUAL( "D41D8CD98F00B204E9800998ECF8427E", run_test_md5( "" ) )
	TESTLY_ASSERT_STRING_EQUAL( "4D52535C7692376E7E7E205940A93AE9", run_test_md5( "945399884.61923487334tuvga" ) )
	TESTLY_ASSERT_STRING_EQUAL( "900150983CD24FB0D6963F7D28E17F72", run_test_md5( "abc" ) )
	TESTLY_ASSERT_STRING_EQUAL( "F96B697D7CB7938D525A2F31AAF161D0", run_test_md5( "message digest" ) )
	TESTLY_ASSERT_STRING_EQUAL( "C3FCD3D76192E4007DFB496CCA67E13B", run_test_md5( "abcdefghijklmnopqrstuvwxyz" ) )
	TESTLY_ASSERT_STRING_EQUAL( "D174AB98D277D9F5A5611C2C9F419D9F" ,run_test_md5( "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789" ) )
	TESTLY_ASSERT_STRING_EQUAL( "57EDF4A22BE3C955AC49DA2E2107B67A", _
							run_test_md5( "12345678901234567890123456789012345678901234567890123456789012345678901234567890" ) )

	end sub
	
	
	private sub register constructor
		ext.testly.addSuite("ext-hashes")
		ext.testly.addTest("test_adler32", @test_adler32)
		ext.testly.addTest("test_crc32", @test_crc32)
		ext.testly.addTest("test_joaat", @test_joaat32)
		ext.testly.addTest("test_joaat64", @test_joaat64)
		ext.testly.addTest("test_md5", @test_md5)
	end sub

end namespace
